package com.mgmtp.insurance.web.component.xform.include;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.mgmtp.insurance.services.claim.ClaimService;

/**
 * This wrapper buffers all written bytes into a ByteArrayServletOutputStream.
 */
public class BufferingHttpServletResponseWrapper extends
		HttpServletResponseWrapper {
	ByteArrayServletOutputStream out;

	private static Log log = LogFactory.getLog(BufferingHttpServletResponseWrapper.class);
	
	/**
	 * GenericResponseWrapper constructor
	 * 
	 * @param response HttpServletResponse
	 */
	public BufferingHttpServletResponseWrapper( HttpServletResponse response) {
		super( response);
		this.out = new ByteArrayServletOutputStream();
	}

	public byte[] getBytes() {
		return out.getBytes();
	}

	/**
	 * getOutputStream overridden method to capture the output written to the
	 * ServletOutputStream Referring to Java Servlet Specification 2.4 this
	 * method throws an IllegalStateException if the @see getWriter method has
	 * been called.
	 */
	@Override
	public ServletOutputStream getOutputStream() throws IOException {
		return out;
	}

	/**
	 * Gets the PrintWriter to write data to. Referring to Java Servlet
	 * Specification 2.4 this method throws an IllegalStateException if the @see
	 * getOutputStream method has been called. In addition the writer uses the
	 * character encoding returned by getCharacterEncoding().
	 * 
	 * @return PrintWriter
	 */
	@Override
	public PrintWriter getWriter() throws IOException {
		throw new IllegalStateException( getClass().getSimpleName()
				+ " only supports getOutputStream()");
	}

	@Override
	public void setContentLength( int len) {
		System.out.println( "ignoring setContentLength " + len);
		// super.setContentLength( len);
	}

	@Override
	public void flushBuffer() throws IOException {
		out.flush();
	}

	/**
	 * This is needed because the underlying Orbeon logic does a forward() and
	 * the RequestDispatcher would throw an IllegalStateException, because it
	 * assumes, that output has already been written to the client.
	 * 
	 * This ServletResponse never writes to the client because all output is
	 * buffered in a byte[] - therefore it should be safe to always say false.
	 */
	@Override
	public boolean isCommitted() {
		return false;
	}

	/**
	 * We must intercept this, because Orbeon expects that no one else writes to
	 * the ServletOutputStream. But because we embed the Orbeon output in JSP,
	 * this would thow an IllegalStateException.
	 * 
	 * This ServletResponse never writes to the client because all output is
	 * buffered in a byte[] - therefore it should be safe to just ignore this.
	 */
	@Override
	public void reset() {
	}
	
	/** See above... */
	@Override
	public void resetBuffer() {
	}
}
