package com.mgmtp.insurance.web.component.xform.include;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;

import java.io.InputStreamReader;
import java.io.Reader;
import java.net.URL;
import java.net.URLConnection;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * A new component to display an xform. 
 * 
 * @author Jan Matz
 * @date $LastChangedDate$
 * @revision $LastChangedRevision$
 */
public class IncludeXFormTagRendererHTTP extends Renderer {

    private static final String XFORM_SRC_FOLDER = "/xforms/";

    /**
     * IceFaces component that represents an Orbeon XForms. As it seems,
     * IceFaces is unable to use the RequestDispatcher.include method. Therefore
     * we do a manual HTTP request to obtain the transformed XForms.
     */
    public void encodeEnd( FacesContext context, UIComponent component )
            throws IOException {

        InputStream is = null;
        try {

            IncludeXFormTag xFormTag = getIncludeXFormTagComponent( component );

            //Only render the component if the rendered attribute is set.
            if( xFormTag.isRendered() ) {

                HttpServletRequest request =
                        (HttpServletRequest) context.getExternalContext().getRequest();
                HttpServletResponse response =
                        (HttpServletResponse) context.getExternalContext().getResponse();

                //Get the writer responsible for outputting the html.
                ResponseWriter writer = context.getResponseWriter();

                //Read the requestURL and remove serverURL in order to retrieve the base URL.
                String serverUrl = request.getRequestURL().toString();
                serverUrl = serverUrl.replace( request.getServletPath(), "" );

                //Read the template provided in the component tag
                String form = xFormTag.getTemplate();

                //Generate URL to the xforms template file.
                String requestURL = serverUrl + XFORM_SRC_FOLDER + form; // + (form.indexOf( '?') >= 0 ? "&" : "?")
                //+ "orbeon-embeddable=true";

                System.out.println( "IncludeFormTag: dispatching to " + requestURL );

                //Add javascript required by the orbeon xforms field.
//				writer.startElement(HTML.SCRIPT_ELEM, component);
//				writer.write("var javaScriptLoadingIntervalID = window.setInterval(function() {" +
//							"if (typeof ORBEON != \"undefined\") {" +
//							"window.clearInterval(javaScriptLoadingIntervalID); " +
//							"ORBEON.xforms.Init.document(); " +
//							"ORBEON.xforms.Globals.baseURL = \"/orbeon\"; " +
//							"ORBEON.xforms.Globals.xformsServerURL = \"/orbeon/xforms-server\";}" +
//							"}, 10);");
//				writer.endElement(HTML.SCRIPT_ELEM);

                // flush previously written output if requested, because
                // BufferingHttpServletResponseWrapper doesn't write to the same
                // output stream as JSP. Otherwise the output could get corrupted.
                writer.flush();

                // not supported by JSF
//                request.getRequestDispatcher( form).include( request, response);

                if( request.getQueryString()!=null) {
                    if( requestURL.contains( "?")) {
                        requestURL += "&" + request.getQueryString();
                    } else {
                        requestURL += "?" + request.getQueryString();
                    }
                }

                System.out.println( "IncludeXFormTagRenderer: " + requestURL);
                URL url = new URL( requestURL );
                URLConnection conn = url.openConnection();

                Cookie[] cookies = request.getCookies();
                if( cookies != null ) {
                    StringBuilder sb = new StringBuilder();
                    for( int i = 0; i < cookies.length; i++ ) {
                        sb.append( cookies[i].getName()).append( "=").append( cookies[ i].getValue());
                        if( i<cookies.length-1) {
                            sb.append( "; ");
                        }
                    }
                    conn.setRequestProperty( "Cookie", sb.toString());
                }

                // Close any open form tag (e.g. rendered by IceFaces)
                // Orbeon renders it's own form tag, which it needs to function
                writer.write( "</form>");

                // Obtain output stream
                is = conn.getInputStream();
                Reader in = new BufferedReader( new InputStreamReader(
                        new BufferedInputStream( is ), "UTF-8" ) );
                int i = 0;
                while( (i = in.read()) >= 0 ) {
                    writer.write( (char) i );
                }

                writer.flush();
            }
        } catch( Exception se ) {
            throw new IOException( se );
        } finally {
            if( is != null ) {
                is.close();
            }
        }
    }

    /**
     * Get the IncludeXFormTag component from the UIComponent.
     * @param component a UIComponent
     * @return IncludeXFormTag component
     */
    protected IncludeXFormTag getIncludeXFormTagComponent( UIComponent component ) {
        IncludeXFormTag value = null;
        if( component instanceof IncludeXFormTag ) {
            value = (IncludeXFormTag) component;
        }

        return value;
    }
}
