package com.mgmtp.insurance.web.component.xform.include;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;

import java.io.InputStreamReader;
import java.io.Reader;
import java.net.URL;
import java.net.URLConnection;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * A new component to display an xform. 
 * 
 * @author Jan Matz
 * @date $LastChangedDate$
 * @revision $LastChangedRevision$
 */
public class IncludeXFormTagRendererIFrame extends Renderer {

    /**
     * IceFaces component that represents an Orbeon XForms. As it seems,
     * IceFaces is unable to use the RequestDispatcher.include method. Therefore
     * we do a manual HTTP request to obtain the transformed XForms.
     */
    public void encodeEnd( FacesContext context, UIComponent component )
            throws IOException {

        try {

            IncludeXFormTag xFormTag = getIncludeXFormTagComponent( component );

            //Only render the component if the rendered attribute is set.
            if( xFormTag.isRendered() ) {

                HttpServletRequest request =
                        (HttpServletRequest) context.getExternalContext().getRequest();
                HttpServletResponse response =
                        (HttpServletResponse) context.getExternalContext().getResponse();

                //Get the writer responsible for outputting the html.
                ResponseWriter writer = context.getResponseWriter();

                //Read the template provided in the component tag
                String form = xFormTag.getTemplate();

                //Generate URL to the xforms template file.
                String iframeLink = form;

                if( request.getQueryString()!=null) {
                    if( iframeLink.contains( "?")) {
                        iframeLink += "&" + request.getQueryString();
                    } else {
                        iframeLink += "?" + request.getQueryString();
                    }
                }

                writer.write( "<iframe id=\"xformsframe\" " +
                        "name=\"xformsframe\" " +
                        "class=\"autoHeight\" " +
                        "width=\"100%\" " +
                        "scrolling=\"no\" " +
                        "frameborder=\"0\" " +
                        "src=\"" + iframeLink + "\"></iframe>");

            }
        } catch( Exception se ) {
            throw new IOException( se );
        }
    }

    /**
     * Get the IncludeXFormTag component from the UIComponent.
     * @param component a UIComponent
     * @return IncludeXFormTag component
     */
    protected IncludeXFormTag getIncludeXFormTagComponent( UIComponent component ) {
        IncludeXFormTag value = null;
        if( component instanceof IncludeXFormTag ) {
            value = (IncludeXFormTag) component;
        }

        return value;
    }
}
