package com.mgmtp.insurance.web.component.xform.include;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;

import java.io.InputStreamReader;
import java.io.Reader;
import java.net.URL;
import java.net.URLConnection;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletContext;
import javax.servlet.ServletResponse;
import javax.servlet.ServletResponseWrapper;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * A new component to display an xform. 
 * 
 * @author Jan Matz
 * @date $LastChangedDate$
 * @revision $LastChangedRevision$
 */
public class IncludeXFormTagRendererInterServlet extends Renderer {

    private static final String XFORM_SRC_FOLDER = "/xforms/";

    /**
     * IceFaces component that represents an Orbeon XForms. As it seems,
     * IceFaces is unable to use the RequestDispatcher.include method. Therefore
     * we do a manual HTTP request to obtain the transformed XForms.
     */
    public void encodeEnd( FacesContext context, UIComponent component )
            throws IOException {

        InputStream is = null;
        try {

            IncludeXFormTag xFormTag = getIncludeXFormTagComponent( component );

            //Only render the component if the rendered attribute is set.
            if( xFormTag.isRendered() ) {

                HttpServletRequest request =
                        (HttpServletRequest) context.getExternalContext().getRequest();
                HttpServletResponse response =
                        (HttpServletResponse) context.getExternalContext().getResponse();

                //Get the writer responsible for outputting the html.
                ResponseWriter writer = context.getResponseWriter();

//                // flush previously written output if requested, because
//                // BufferingHttpServletResponseWrapper doesn't write to the same
//                // output stream as JSP. Otherwise the output could get corrupted.
                writer.flush();
                
                writer.write("</FORM>");
                writer.write("<DIV>");
                writer.flush();
                
                //Read the template provided in the component tag
                String form = xFormTag.getTemplate();

                //Generate URL to the xforms template file.
                String requestURL = XFORM_SRC_FOLDER + form;

                System.out.println( "IncludeFormTag: dispatching to " + requestURL );
                
                BufferingHttpServletResponseWrapper wrappedResponse = 
                	new BufferingHttpServletResponseWrapper(response);
                
                ServletContext servlet = (ServletContext)context.getExternalContext().getContext();
                servlet.getRequestDispatcher(requestURL).include(request, wrappedResponse);
                
                for(Byte b : wrappedResponse.getBytes()){
                	writer.write(b.intValue());
                }
                
                writer.write("</DIV>");
                writer.flush();
            }
        } catch( Exception se ) {
            throw new IOException( se );
        } finally {
            if( is != null ) {
                is.close();
            }
        }
    }
    
    /**
     * Get the IncludeXFormTag component from the UIComponent.
     * @param component a UIComponent
     * @return IncludeXFormTag component
     */
    protected IncludeXFormTag getIncludeXFormTagComponent( UIComponent component ) {
        IncludeXFormTag value = null;
        if( component instanceof IncludeXFormTag ) {
            value = (IncludeXFormTag) component;
        }

        return value;
    }
}
